function y = islantltA(s)
%  islantltA  inverse slantlet transform, "another" program.
%
%  This program does not require the vectors u0 and u1.
%
%  See also slantlt, sislet, isislet, sltmtx.
%
%  % example
%  x = sin(sin((1:2^5)/3));
%  s = slantlt(x);
%  y = islantltA(s);
%  max(abs(x-y))

%  Ivan Selesnick, 1997
%  subprograms: getg.m, gethf.m


N = length(s);
l = log2(N);
if l ~= round(l)
    disp('sorry, need length(s) a power of 2.')
    y = [];
    return
end

y = zeros(size(s));

m = 2^l;
y(1) = s(1)/sqrt(m) + s(2)*sqrt(3*(m-1)/(m*(m+1)));   % dc component
y(2) = -s(2)*2*sqrt(3/(m*(m^2-1)));                   % linear component

L = N/2;
for i = 1:l-1
    n0 = 1:2*L:N;
    n1 = n0+L;
    L = L/2;
    y(n1) = y(n0)+2^(l-i)*y(n0+1);
    y(n1+1) = y(n0+1);

    [a0,a1,b0,b1,a0r,a1r,b0r,b1r] = getg(l-i);

    k0 = 2^i+1:2:2^(i+1);
    k1 = k0+1;
    y(n0) = y(n0) + a0*s(k0) + a0r*s(k1);
    y(n1) = y(n1) + b0*s(k0) + b0r*s(k1);
    y(n0+1) = y(n0+1) + a1*s(k0) + a1r*s(k1);
    y(n1+1) = y(n1+1) + b1*s(k0) + b1r*s(k1);
end

y(2:2:N) = y(1:2:N)+y(2:2:N);
