function y = islet2(s)
%
% inverse two-dimensional slantlet transform
%
% % example
% x = randn(2^5);
% s = slet2(x);
% y = islet2(s);
% max(max(abs(x-y)))
%

N = length(s);
l = log2(N);
if l ~= round(l)
    disp('sorry, need length(s) a power of 2.')
    y = [];
    return
end


u00 = zeros(N/2);
u01 = zeros(N/2);
u10 = zeros(N/2);
u11 = zeros(N/2);

m = 2^l;
c1 = sqrt(3*(m-1)/(m*(m+1)));
c2 = (-2*sqrt(3/(m*(m^2-1))));

u00(1) = s(1,1)/m   + s(1,2)*c1/sqrt(m) + s(2,1)*c1/sqrt(m) + s(2,2)*c1^2;
u01(1) = s(1,2)*c2/sqrt(m) + s(2,2)*c1*c2;
u10(1) = s(2,1)*c2/sqrt(m) + s(2,2)*c1*c2;
u11(1) = s(2,2)*c2^2;


L = N/2;
for i = 1:l-1
    n0 = 1:L:N/2;
    L = L/2;
    n1 = n0+L;

    u00(n0,n1) = u00(n0,n0) + 2^(l-i)*u01(n0,n0);
    u01(n0,n1) = u01(n0,n0);
    u10(n0,n1) = u10(n0,n0) + 2^(l-i)*u11(n0,n0);
    u11(n0,n1) = u11(n0,n0);

    u00(n1,n0) = u00(n0,n0) + 2^(l-i)*u10(n0,n0);
    u01(n1,n0) = u01(n0,n0) + 2^(l-i)*u11(n0,n0);
    u10(n1,n0) = u10(n0,n0);
    u11(n1,n0) = u11(n0,n0);

    u00(n1,n1) = u00(n0,n0) + 2^(l-i)*u01(n0,n0) + 2^(l-i)*u10(n0,n0) + 2^(2*l-2*i)*u11(n0,n0);
    u01(n1,n1) = u01(n0,n0) + 2^(l-i)*u11(n0,n0);
    u10(n1,n1) = u10(n0,n0) + 2^(l-i)*u11(n0,n0);
    u11(n1,n1) = u11(n0,n0);

    [a0,a1,b0,b1,a0r,a1r,b0r,b1r] = getg(l-i);
    [c0,c1,c0r,c1r] = gets(l-i);

    d0 = c0 + c1*(2^(l-i));
    d1 = c1;

    d0r = c0r + c1r*(2^(l-i));
    d1r = c1r;

    k0 = 2^i+1:2:2^(i+1);
    k1 = k0+1;

    % hi-hi (g'*g)

    tmp1 = a0*s(k0,k0) + a0r*s(k0,k1);
    tmp2 = a0*s(k1,k0) + a0r*s(k1,k1);
    tmp3 = a1*s(k0,k0) + a1r*s(k0,k1);
    tmp4 = a1*s(k1,k0) + a1r*s(k1,k1);

    u00(n0,n0) = u00(n0,n0) + a0*tmp1 + a0r*tmp2;
    u01(n0,n0) = u01(n0,n0) + a0*tmp3 + a0r*tmp4;
    u10(n0,n0) = u10(n0,n0) + a1*tmp1 + a1r*tmp2;
    u11(n0,n0) = u11(n0,n0) + a1*tmp3 + a1r*tmp4;

    u00(n1,n0) = u00(n1,n0) + b0*tmp1 + b0r*tmp2;
    u01(n1,n0) = u01(n1,n0) + b0*tmp3 + b0r*tmp4;
    u10(n1,n0) = u10(n1,n0) + b1*tmp1 + b1r*tmp2;
    u11(n1,n0) = u11(n1,n0) + b1*tmp3 + b1r*tmp4;

    tmp1 = b0*s(k0,k0) + b0r*s(k0,k1);
    tmp2 = b0*s(k1,k0) + b0r*s(k1,k1);
    tmp3 = b1*s(k0,k0) + b1r*s(k0,k1);
    tmp4 = b1*s(k1,k0) + b1r*s(k1,k1);

    u00(n0,n1) = u00(n0,n1) + a0*tmp1 + a0r*tmp2;
    u01(n0,n1) = u01(n0,n1) + a0*tmp3 + a0r*tmp4;
    u10(n0,n1) = u10(n0,n1) + a1*tmp1 + a1r*tmp2;
    u11(n0,n1) = u11(n0,n1) + a1*tmp3 + a1r*tmp4;

    u00(n1,n1) = u00(n1,n1) + b0*tmp1 + b0r*tmp2;
    u01(n1,n1) = u01(n1,n1) + b0*tmp3 + b0r*tmp4;
    u10(n1,n1) = u10(n1,n1) + b1*tmp1 + b1r*tmp2;
    u11(n1,n1) = u11(n1,n1) + b1*tmp3 + b1r*tmp4;

    v0 = 1:2:2^i;
    v1 = v0 + 1;

    % low-hi (s'*g)

    tmp1 = a0*s(k0,v0) + a0r*s(k0,v1);
    tmp2 = a0*s(k1,v0) + a0r*s(k1,v1);
    tmp3 = a1*s(k0,v0) + a1r*s(k0,v1);
    tmp4 = a1*s(k1,v0) + a1r*s(k1,v1);

    u00(n0,n0) = u00(n0,n0) + c0*tmp1 + c0r*tmp2;
    u01(n0,n0) = u01(n0,n0) + c0*tmp3 + c0r*tmp4;
    u10(n0,n0) = u10(n0,n0) + c1*tmp1 + c1r*tmp2;
    u11(n0,n0) = u11(n0,n0) + c1*tmp3 + c1r*tmp4;

    u00(n1,n0) = u00(n1,n0) + d0*tmp1 + d0r*tmp2;
    u01(n1,n0) = u01(n1,n0) + d0*tmp3 + d0r*tmp4;
    u10(n1,n0) = u10(n1,n0) + d1*tmp1 + d1r*tmp2;
    u11(n1,n0) = u11(n1,n0) + d1*tmp3 + d1r*tmp4;

    tmp1 = b0*s(k0,v0) + b0r*s(k0,v1);
    tmp2 = b0*s(k1,v0) + b0r*s(k1,v1);
    tmp3 = b1*s(k0,v0) + b1r*s(k0,v1);
    tmp4 = b1*s(k1,v0) + b1r*s(k1,v1);

    u00(n0,n1) = u00(n0,n1) + c0*tmp1 + c0r*tmp2;
    u01(n0,n1) = u01(n0,n1) + c0*tmp3 + c0r*tmp4;
    u10(n0,n1) = u10(n0,n1) + c1*tmp1 + c1r*tmp2;
    u11(n0,n1) = u11(n0,n1) + c1*tmp3 + c1r*tmp4;

    u00(n1,n1) = u00(n1,n1) + d0*tmp1 + d0r*tmp2;
    u01(n1,n1) = u01(n1,n1) + d0*tmp3 + d0r*tmp4;
    u10(n1,n1) = u10(n1,n1) + d1*tmp1 + d1r*tmp2;
    u11(n1,n1) = u11(n1,n1) + d1*tmp3 + d1r*tmp4;

    % hi-low (g'*s)

    tmp1 = c0*s(v0,k0) + c0r*s(v0,k1);
    tmp2 = c0*s(v1,k0) + c0r*s(v1,k1);
    tmp3 = c1*s(v0,k0) + c1r*s(v0,k1);
    tmp4 = c1*s(v1,k0) + c1r*s(v1,k1);

    u00(n0,n0) = u00(n0,n0) + a0*tmp1 + a0r*tmp2;
    u01(n0,n0) = u01(n0,n0) + a0*tmp3 + a0r*tmp4;
    u10(n0,n0) = u10(n0,n0) + a1*tmp1 + a1r*tmp2;
    u11(n0,n0) = u11(n0,n0) + a1*tmp3 + a1r*tmp4;

    u00(n1,n0) = u00(n1,n0) + b0*tmp1 + b0r*tmp2;
    u01(n1,n0) = u01(n1,n0) + b0*tmp3 + b0r*tmp4;
    u10(n1,n0) = u10(n1,n0) + b1*tmp1 + b1r*tmp2;
    u11(n1,n0) = u11(n1,n0) + b1*tmp3 + b1r*tmp4;

    tmp1 = d0*s(v0,k0) + d0r*s(v0,k1);
    tmp2 = d0*s(v1,k0) + d0r*s(v1,k1);
    % tmp3 = d1*s(v0,k0) + d1r*s(v0,k1);    % tmp3 and tmp4 do not need updating because d1 = c1
    % tmp4 = d1*s(v1,k0) + d1r*s(v1,k1);

    u00(n0,n1) = u00(n0,n1) + a0*tmp1 + a0r*tmp2;
    u01(n0,n1) = u01(n0,n1) + a0*tmp3 + a0r*tmp4;
    u10(n0,n1) = u10(n0,n1) + a1*tmp1 + a1r*tmp2;
    u11(n0,n1) = u11(n0,n1) + a1*tmp3 + a1r*tmp4;

    u00(n1,n1) = u00(n1,n1) + b0*tmp1 + b0r*tmp2;
    u01(n1,n1) = u01(n1,n1) + b0*tmp3 + b0r*tmp4;
    u10(n1,n1) = u10(n1,n1) + b1*tmp1 + b1r*tmp2;
    u11(n1,n1) = u11(n1,n1) + b1*tmp3 + b1r*tmp4;

end


y = zeros(N);
y(1:2:N,1:2:N) = u00;
y(1:2:N,2:2:N) = u00+u01;
y(2:2:N,1:2:N) = u00+u10;
y(2:2:N,2:2:N) = u00+u01+u10+u11;
